#!/bin/bash

# This code is under GPL:
# http://www.gnu.org/copyleft/gpl.html

ROOTIMG=root.imz
BOOTIMG=boot.img
FLOPPYDEV=${FLOPPYDEV:-/dev/fd0}  # floppy device to create boot disk
LOOPDEV=${LOOPDEV:-/dev/loop0}    # loop device which should be used

# Create root image (initrd)
function root () {
	if grep /dev/ram0 /proc/mounts; then
		umount /dev/ram0
	fi
	test ! -d root.mnt && mkdir root.mnt
	{
	echo "** Creating image" &&
	dd if=/dev/zero of=/dev/ram0 bs=1k count=4096 &&
	mke2fs /dev/ram0 -m 0 >/dev/null &&
	mount /dev/ram0  root.mnt/ &&
	rm -rf root/lib/modules
	cp kernel/modules/lib/modules root/lib/ -a
	cp root/* root.mnt/ -a &&
	sync &&
	echo "** Shared Libs on" &&
	ldconfig -v -r root.mnt &&
	umount root.mnt &&
	echo "** Compacting image" &&
	dd if=/dev/ram0 | gzip -c9 > $ROOTIMG
	rm -rf root.mnt
	echo "** Root image created as $ROOTIMG"
	} || { rm -rf root.mnt; false; }

}

# Create boot image
function boot () {

	# this function was gracefully stoled (but littel modified)
	# from DLX Linux mini distro

	RD_KERNEL=kernel/bzImage
	RD_IMAGE=$ROOTIMG

	# here we make some calculations

	KERNELSIZE=$(ls -Ls $RD_KERNEL | awk '{print $1}')
	IMAGESIZE=$(ls -Ls $RD_IMAGE | awk '{print $1}')

	LOOPTABLE=$[$KERNELSIZE + 1]
	BOOTSTART=$[$KERNELSIZE + 2]
	LOOPSTART=$[1024 * ($BOOTSTART + $IMAGESIZE)]

	# if you want the kernel to ask for the boot dev
	# add 32768 to the RAMDISKFLAG

	RD_FLAG=$[$BOOTSTART + 16384]

	#########################################
	#
	# prg starts here 

	cat <<-EOF
	** Writing boot image to disk ...
	** --------------------------------
	** kernelsize: $KERNELSIZE
	** bootstart : $BOOTSTART
	** loopstart : $LOOPSTART
	** tablestart: $LOOPTABLE
	** Floppy: $FLOPPYDEV
	** Loop: $LOOPDEV
	** --------------------------------
	EOF

	echo "** Writing kernel: $RD_KERNEL"
	rdev $RD_KERNEL $FLOPPYDEV     # set boot device
	rdev -r $RD_KERNEL $RD_FLAG    # set start of ramdisk
	rdev -R $RD_KERNEL 0           # default VGA !
	dd if=$RD_KERNEL of=$FLOPPYDEV bs=512  # copy kernel to disk

	echo "** Writing image: $RD_IMAGE"
	dd if=$RD_IMAGE of=$FLOPPYDEV bs=1k seek=$BOOTSTART

	echo "** Creating loop FS"
	echo "$LOOPSTART" | dd of=$FLOPPYDEV bs=1k seek=$LOOPTABLE conv=sync
	losetup $LOOPDEV $FLOPPYDEV -o $LOOPSTART
	mke2fs -m 0 $LOOPDEV > /dev/null
	losetup -d $LOOPDEV
	sync

	echo
	echo "** Boot disk created on $FLOPPYDEV."

}

case $1 in
	boot) boot;;
	root) root;;
	all) root; boot;;
	*) cat <<-EOF
		Use: $0 <boot|root|all>

		root:  create root image only (initrd)";
		  boot:  create boot disk with root image from $ROOTIMG";
		  all:   create root and boot images.
		  Same as "# $0 root; $0 boot"
		
		Things you could export before create the boot disk:

		  # export FLOPPYDEV=/dev/fd0
		  # export LOOPDEV=/dev/loop0
EOF
esac

